@@ -3,39 +3,53 @@
     tests.storage
     ~~~~~~~~~~~~~
 
-    :copyright: (c) 2014 Markus Unterwaditzer
+    :copyright: (c) 2014 Markus Unterwaditzer & contributors
     :license: MIT, see LICENSE for more details.
 '''
-
-from vdirsyncer.storage.base import Item
-import vdirsyncer.exceptions as exceptions
-from vdirsyncer.utils import text_type
-from .. import assert_item_equals
 import random
+
 import pytest
 
+import vdirsyncer.exceptions as exceptions
+from vdirsyncer.storage.base import Item
+from vdirsyncer.utils.compat import iteritems, text_type
 
-class StorageTests(object):
-    item_template = u'UID:{uid}\nX-SOMETHING:{r}'
+from .. import SIMPLE_TEMPLATE, assert_item_equals
 
-    def _create_bogus_item(self, uid, item_template=None):
-        r = random.random()
-        item_template = item_template or self.item_template
-        return Item(item_template.format(uid=uid, r=r))
+
+class BaseStorageTests(object):
+    item_template = SIMPLE_TEMPLATE
+
+    @pytest.fixture
+    def storage_args(self):
+        return self.get_storage_args
 
     def get_storage_args(self, collection=None):
         raise NotImplementedError()
 
-    def _get_storage(self):
-        s = self.storage_class(**self.get_storage_args())
-        assert not list(s.list())
-        return s
+    @pytest.fixture
+    def get_storage(self, storage_args):
+        def inner(**kw):
+            return self.storage_class(**storage_args(**kw))
 
-    def test_generic(self):
-        items = map(self._create_bogus_item, range(1, 10))
-        for i, item in enumerate(items):
-            assert item.uid == text_type(i + 1), item.raw
-        s = self._get_storage()
+        return inner
+
+    @pytest.fixture
+    def s(self, get_storage):
+        return get_storage()
+
+    @pytest.fixture
+    def get_item(self):
+        def inner(item_template=None):
+            # assert that special chars are handled correctly.
+            r = '{}@vdirsyncer'.format(random.random())
+            item_template = item_template or self.item_template
+            return Item(item_template.format(r=r))
+
+        return inner
+
+    def test_generic(self, s, get_item):
+        items = [get_item() for i in range(1, 10)]
         hrefs = []
         for item in items:
             hrefs.append(s.upload(item))
@@ -47,70 +61,85 @@ class StorageTests(object):
             assert s.has(href)
             item, etag2 = s.get(href)
             assert etag == etag2
-            assert 'UID:{}'.format(item.uid) in item.raw
 
-    def test_empty_get_multi(self):
-        s = self._get_storage()
+    def test_empty_get_multi(self, s):
         assert list(s.get_multi([])) == []
 
-    def test_upload_already_existing(self):
-        s = self._get_storage()
-        item = self._create_bogus_item(1)
+    def test_upload_already_existing(self, s, get_item):
+        item = get_item()
         s.upload(item)
         with pytest.raises(exceptions.PreconditionFailed):
             s.upload(item)
 
-    def test_upload(self):
-        s = self._get_storage()
-        item = self._create_bogus_item(1)
+    def test_upload(self, s, get_item):
+        item = get_item()
         href, etag = s.upload(item)
         assert_item_equals(s.get(href)[0], item)
 
-    def test_update(self):
-        s = self._get_storage()
-        item = self._create_bogus_item(1)
+    def test_update(self, s, get_item):
+        item = get_item()
         href, etag = s.upload(item)
         assert_item_equals(s.get(href)[0], item)
 
-        new_item = self._create_bogus_item(1)
-        s.update(href, new_item, etag)
+        new_item = get_item()
+        new_etag = s.update(href, new_item, etag)
+        # See https://github.com/untitaker/vdirsyncer/issues/48
+        assert isinstance(new_etag, (bytes, text_type))
         assert_item_equals(s.get(href)[0], new_item)
 
-    def test_update_nonexisting(self):
-        s = self._get_storage()
-        item = self._create_bogus_item(1)
-        with pytest.raises(exceptions.PreconditionFailed):
-            s.update(s._get_href('1'), item, '"123"')
+    def test_update_nonexisting(self, s, get_item):
+        item = get_item()
         with pytest.raises(exceptions.PreconditionFailed):
             s.update('huehue', item, '"123"')
 
-    def test_wrong_etag(self):
-        s = self._get_storage()
-        item = self._create_bogus_item(1)
+    def test_wrong_etag(self, s, get_item):
+        item = get_item()
         href, etag = s.upload(item)
         with pytest.raises(exceptions.PreconditionFailed):
             s.update(href, item, '"lolnope"')
         with pytest.raises(exceptions.PreconditionFailed):
             s.delete(href, '"lolnope"')
 
-    def test_delete(self):
-        s = self._get_storage()
-        href, etag = s.upload(self._create_bogus_item(1))
+    def test_delete(self, s, get_item):
+        href, etag = s.upload(get_item())
         s.delete(href, etag)
         assert not list(s.list())
 
-    def test_delete_nonexisting(self):
-        s = self._get_storage()
+    def test_delete_nonexisting(self, s, get_item):
         with pytest.raises(exceptions.PreconditionFailed):
             s.delete('1', '"123"')
 
-    def test_list(self):
-        s = self._get_storage()
+    def test_list(self, s, get_item):
         assert not list(s.list())
-        s.upload(self._create_bogus_item('1'))
-        assert list(s.list())
+        href, etag = s.upload(get_item())
+        assert list(s.list()) == [(href, etag)]
 
-    def test_discover(self):
+    def test_has(self, s, get_item):
+        assert not s.has('asd')
+        href, etag = s.upload(get_item())
+        assert s.has(href)
+        assert not s.has('asd')
+
+    def test_update_others_stay_the_same(self, s, get_item):
+        info = dict([
+            s.upload(get_item()),
+            s.upload(get_item()),
+            s.upload(get_item()),
+            s.upload(get_item())
+        ])
+
+        assert dict(
+            (href, etag) for href, item, etag
+            in s.get_multi(href for href, etag in iteritems(info))
+        ) == info
+
+    def test_repr(self, s):
+        assert self.storage_class.__name__ in repr(s)
+
+
+class SupportsCollections(object):
+
+    def test_discover(self, storage_args, get_item):
         collections = set()
 
         def main():
@@ -119,18 +148,17 @@ class StorageTests(object):
                 # Create collections on-the-fly for most storages
                 # Except ownCloud, which already has all of them, and more
                 i += 1
-                s = self.storage_class(
-                    **self.get_storage_args(collection=collection))
+                s = self.storage_class(**storage_args(collection=collection))
 
                 # radicale ignores empty collections during discovery
-                item = self._create_bogus_item(str(i))
+                item = get_item()
                 s.upload(item)
 
                 collections.add(s.collection)
         main()  # remove leftover variables from loop for safety
 
         d = self.storage_class.discover(
-            **self.get_storage_args(collection=None))
+            **storage_args(collection=None))
 
         def main():
             for s in d:
@@ -145,22 +173,19 @@ class StorageTests(object):
 
         assert not collections
 
-    def test_discover_collection_arg(self):
-        args = self.get_storage_args(collection='test2')
+    def test_discover_collection_arg(self, storage_args):
+        args = storage_args(collection='test2')
         with pytest.raises(TypeError) as excinfo:
             list(self.storage_class.discover(**args))
 
         assert 'collection argument must not be given' in str(excinfo.value)
 
-    def test_collection_arg(self):
-        s = self.storage_class(**self.get_storage_args(collection='test2'))
+    def test_collection_arg(self, get_storage):
+        s = get_storage(collection='test2')
         # Can't do stronger assertion because of radicale, which needs a
         # fileextension to guess the collection type.
         assert 'test2' in s.collection
 
-    def test_has(self):
-        s = self._get_storage()
-        assert not s.has('asd')
-        href, etag = s.upload(self._create_bogus_item(1))
-        assert s.has(href)
-        assert not s.has('asd')
+
+class StorageTests(BaseStorageTests, SupportsCollections):
+    pass
