@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
 
-import sys
 import os
-import pytest
-from vdirsyncer.utils import urlparse
+import sys
 
-from werkzeug.test import Client
-from werkzeug.wrappers import BaseResponse as WerkzeugResponse
+import pytest
+
+import wsgi_intercept
+import wsgi_intercept.requests_intercept
+
+wsgi_intercept.requests_intercept.install()
 
 
 RADICALE_SCHEMA = '''
@@ -39,7 +41,7 @@ create table property (
        value text not null,
        collection_path varchar(200) references collection (path),
        primary key (name, collection_path));
-'''
+'''.split(';')
 
 storage_backend = os.environ.get('RADICALE_BACKEND', '') or 'filesystem'
 
@@ -65,8 +67,7 @@ def do_the_radicale_dance(tmpdir):
     import radicale.auth.http
 
     def is_authenticated(user, password):
-        assert user == 'bob' and password == 'bob'
-        return True
+        return user == 'bob' and password == 'bob'
     radicale.auth.http.is_authenticated = is_authenticated
 
     if storage_backend == 'filesystem':
@@ -78,7 +79,7 @@ def do_the_radicale_dance(tmpdir):
         from radicale.storage import database
 
         s = database.Session()
-        for line in RADICALE_SCHEMA.split(';'):
+        for line in RADICALE_SCHEMA:
             s.execute(line)
         s.commit()
     else:
@@ -88,27 +89,15 @@ def do_the_radicale_dance(tmpdir):
 class ServerMixin(object):
 
     @pytest.fixture(autouse=True)
-    def setup(self, monkeypatch, tmpdir):
+    def setup(self, request, tmpdir):
         do_the_radicale_dance(str(tmpdir))
         from radicale import Application
-        app = Application()
-        c = Client(app, WerkzeugResponse)
 
-        from requests import Response
+        wsgi_intercept.add_wsgi_intercept('127.0.0.1', 80, Application)
 
-        def send(self, request, *args, **kwargs):
-            path = urlparse.urlparse(request.url).path
-            wr = c.open(path=path, method=request.method,
-                        data=request.body, headers=dict(request.headers))
-            r = Response()
-            r.request = request
-            r._content = wr.get_data(as_text=False)
-            r.headers = wr.headers
-            r.encoding = wr.charset
-            r.status_code = wr.status_code
-            return r
-
-        monkeypatch.setattr('requests.adapters.HTTPAdapter.send', send)
+        def teardown():
+            wsgi_intercept.remove_wsgi_intercept('127.0.0.1', 80)
+        request.addfinalizer(teardown)
 
     def get_storage_args(self, collection='test'):
         url = 'http://127.0.0.1/bob/'
@@ -116,4 +105,4 @@ class ServerMixin(object):
             collection += self.storage_class.fileext
 
         return {'url': url, 'username': 'bob', 'password': 'bob',
-                'collection': collection}
+                'collection': collection, 'unsafe_href_chars': ''}
