@@ -3,23 +3,28 @@
     tests.storage.dav.test_main
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: (c) 2014 Markus Unterwaditzer
+    :copyright: (c) 2014 Markus Unterwaditzer & contributors
     :license: MIT, see LICENSE for more details.
 '''
 
-import os
-import pytest
 import datetime
+import os
 from textwrap import dedent
 
-from .. import StorageTests
-import vdirsyncer.exceptions as exceptions
-from vdirsyncer.storage.base import Item
-from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage
-import vdirsyncer.exceptions
+import pytest
+
 import requests
 import requests.exceptions
 
+from tests import EVENT_TEMPLATE, TASK_TEMPLATE, VCARD_TEMPLATE
+
+import vdirsyncer.exceptions as exceptions
+from vdirsyncer.storage.base import Item
+from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage, \
+    _normalize_href
+
+from .. import StorageTests
+
 
 dav_server = os.environ.get('DAV_SERVER', '').strip() or 'radicale'
 
@@ -31,49 +36,6 @@ def _get_server_mixin(server_name):
 
 ServerMixin = _get_server_mixin(dav_server)
 
-VCARD_TEMPLATE = u'''BEGIN:VCARD
-VERSION:3.0
-FN:Cyrus Daboo
-N:Daboo;Cyrus
-ADR;TYPE=POSTAL:;2822 Email HQ;Suite 2821;RFCVille;PA;15213;USA
-EMAIL;TYPE=INTERNET;TYPE=PREF:cyrus@example.com
-NICKNAME:me
-NOTE:Example VCard.
-ORG:Self Employed
-TEL;TYPE=WORK;TYPE=VOICE:412 605 0499
-TEL;TYPE=FAX:412 605 0705
-URL:http://www.example.com
-UID:{uid}
-X-SOMETHING:{r}
-END:VCARD'''
-
-
-TASK_TEMPLATE = u'''BEGIN:VCALENDAR
-VERSION:2.0
-PRODID:-//dmfs.org//mimedir.icalendar//EN
-BEGIN:VTODO
-CREATED:20130721T142233Z
-DTSTAMP:20130730T074543Z
-LAST-MODIFIED;VALUE=DATE-TIME:20140122T151338Z
-SEQUENCE:2
-SUMMARY:Book: Kowlani - Tödlicher Staub
-UID:{uid}
-X-SOMETHING:{r}
-END:VTODO
-END:VCALENDAR'''
-
-
-EVENT_TEMPLATE = u'''BEGIN:VCALENDAR
-VERSION:2.0
-PRODID:-//hacksw/handcal//NONSGML v1.0//EN
-BEGIN:VEVENT
-DTSTART:19970714T170000Z
-DTEND:19970715T035959Z
-SUMMARY:Bastille Day Party
-X-SOMETHING:{r}
-UID:{uid}
-END:VEVENT
-END:VCALENDAR'''
 
 templates = {
     'VCARD': VCARD_TEMPLATE,
@@ -83,24 +45,21 @@ templates = {
 
 
 class DavStorageTests(ServerMixin, StorageTests):
-    def test_dav_broken_item(self):
-        item = Item(u'UID:1')
-        s = self._get_storage()
+    def test_dav_broken_item(self, s):
+        item = Item(u'HAHA:YES')
         try:
             s.upload(item)
         except (exceptions.Error, requests.exceptions.HTTPError):
             pass
         assert not list(s.list())
 
-    @pytest.mark.xfail(dav_server == 'owncloud',
-                       reason='See issue #16')
-    def test_wrong_etag(self):
-        super(DavStorageTests, self).test_wrong_etag()
+    def test_dav_empty_get_multi_performance(self, s, monkeypatch):
+        def breakdown(*a, **kw):
+            raise AssertionError('Expected not to be called.')
 
-    @pytest.mark.xfail(dav_server == 'owncloud',
-                       reason='See issue #16')
-    def test_update_nonexisting(self):
-        super(DavStorageTests, self).test_update_nonexisting()
+        monkeypatch.setattr('requests.sessions.Session.request', breakdown)
+
+        assert list(s.get_multi([])) == []
 
 
 class TestCaldavStorage(DavStorageTests):
@@ -108,10 +67,9 @@ class TestCaldavStorage(DavStorageTests):
 
     item_template = TASK_TEMPLATE
 
-    def test_both_vtodo_and_vevent(self):
-        task = self._create_bogus_item(1, item_template=TASK_TEMPLATE)
-        event = self._create_bogus_item(2, item_template=EVENT_TEMPLATE)
-        s = self._get_storage()
+    def test_both_vtodo_and_vevent(self, s, get_item):
+        task = get_item(item_template=TASK_TEMPLATE)
+        event = get_item(item_template=EVENT_TEMPLATE)
         href_etag_task = s.upload(task)
         href_etag_event = s.upload(event)
         assert set(s.list()) == set([
@@ -120,20 +78,17 @@ class TestCaldavStorage(DavStorageTests):
         ])
 
     @pytest.mark.parametrize('item_type', ['VTODO', 'VEVENT'])
-    def test_item_types_correctness(self, item_type):
+    def test_item_types_correctness(self, item_type, storage_args, get_item):
         other_item_type = 'VTODO' if item_type == 'VEVENT' else 'VEVENT'
-        kw = self.get_storage_args()
-        s = self.storage_class(item_types=(item_type,), **kw)
+        s = self.storage_class(item_types=(item_type,), **storage_args())
         try:
-            s.upload(self._create_bogus_item(
-                1, item_template=templates[other_item_type]))
-            s.upload(self._create_bogus_item(
-                5, item_template=templates[other_item_type]))
+            s.upload(get_item(item_template=templates[other_item_type]))
+            s.upload(get_item(item_template=templates[other_item_type]))
         except (exceptions.Error, requests.exceptions.HTTPError):
             pass
         href, etag = \
-            s.upload(self._create_bogus_item(
-                3, item_template=templates[item_type]))
+            s.upload(get_item(
+                item_template=templates[item_type]))
         ((href2, etag2),) = s.list()
         assert href2 == href
         assert etag2 == etag
@@ -142,33 +97,39 @@ class TestCaldavStorage(DavStorageTests):
         ('VTODO',),
         ('VEVENT',),
         ('VTODO', 'VEVENT'),
-        ('VTODO', 'VEVENT', 'VJOURNAL')
+        ('VTODO', 'VEVENT', 'VJOURNAL'),
+        ()
     ])
-    def test_item_types_performance(self, item_types, monkeypatch):
-        kw = self.get_storage_args()
-        s = self.storage_class(item_types=item_types, **kw)
+    def test_item_types_performance(self, storage_args, item_types,
+                                    monkeypatch, get_item):
+        s = self.storage_class(item_types=item_types, **storage_args())
+        item = get_item()
+        href, etag = s.upload(item)
 
-        old_list = s._list
+        old_dav_query = s._dav_query
         calls = []
 
-        def _list(*a, **kw):
+        def _dav_query(*a, **kw):
             calls.append(None)
-            return old_list(*a, **kw)
+            return old_dav_query(*a, **kw)
 
-        monkeypatch.setattr(s, '_list', _list)
+        monkeypatch.setattr(s, '_dav_query', _dav_query)
 
-        list(s.list())
-        assert len(calls) == len(item_types)
+        rv = list(s.list())
+        if (dav_server != 'radicale' and not s.item_types) \
+           or item.parsed.name in s.item_types:
+            assert rv == [(href, etag)]
+        assert len(calls) == (len(item_types) or 1)
 
     @pytest.mark.xfail(dav_server == 'radicale',
                        reason='Radicale doesn\'t support timeranges.')
-    def test_timerange_correctness(self):
-        kw = self.get_storage_args()
+    def test_timerange_correctness(self, storage_args, get_item):
         start_date = datetime.datetime(2013, 9, 10)
         end_date = datetime.datetime(2013, 9, 13)
-        s = self.storage_class(start_date=start_date, end_date=end_date, **kw)
+        s = self.storage_class(start_date=start_date, end_date=end_date,
+                               **storage_args())
 
-        too_old_item = self._create_bogus_item('1', item_template=dedent(u'''
+        too_old_item = get_item(item_template=dedent(u'''
             BEGIN:VCALENDAR
             VERSION:2.0
             PRODID:-//hacksw/handcal//NONSGML v1.0//EN
@@ -177,12 +138,12 @@ class TestCaldavStorage(DavStorageTests):
             DTEND:19970715T035959Z
             SUMMARY:Bastille Day Party
             X-SOMETHING:{r}
-            UID:{uid}
+            UID:{r}
             END:VEVENT
             END:VCALENDAR
             ''').strip())
 
-        too_new_item = self._create_bogus_item('2', item_template=dedent(u'''
+        too_new_item = get_item(item_template=dedent(u'''
             BEGIN:VCALENDAR
             VERSION:2.0
             PRODID:-//hacksw/handcal//NONSGML v1.0//EN
@@ -191,12 +152,12 @@ class TestCaldavStorage(DavStorageTests):
             DTEND:20150715T035959Z
             SUMMARY:Another Bastille Day Party
             X-SOMETHING:{r}
-            UID:{uid}
+            UID:{r}
             END:VEVENT
             END:VCALENDAR
             ''').strip())
 
-        good_item = self._create_bogus_item('3', item_template=dedent(u'''
+        good_item = get_item(item_template=dedent(u'''
             BEGIN:VCALENDAR
             VERSION:2.0
             PRODID:-//hacksw/handcal//NONSGML v1.0//EN
@@ -205,7 +166,7 @@ class TestCaldavStorage(DavStorageTests):
             DTEND:20130912T035959Z
             SUMMARY:What's with all these Bastille Day Partys
             X-SOMETHING:{r}
-            UID:{uid}
+            UID:{r}
             END:VEVENT
             END:VCALENDAR
             ''').strip())
@@ -216,19 +177,18 @@ class TestCaldavStorage(DavStorageTests):
 
         assert list(s.list()) == [(href, etag)]
 
-    def test_item_types_passed_as_string(self):
-        kw = self.get_storage_args()
+    def test_item_types_passed_as_string(self, storage_args):
+        kw = storage_args()
         a = self.storage_class(item_types='VTODO,VEVENT', **kw)
         b = self.storage_class(item_types=('VTODO', 'VEVENT'), **kw)
         assert a.item_types == b.item_types == ('VTODO', 'VEVENT')
 
-    def test_invalid_resource(self, monkeypatch):
+    def test_invalid_resource(self, monkeypatch, storage_args):
         calls = []
-        args = self.get_storage_args(collection=None)
+        args = storage_args(collection=None)
 
         def request(session, method, url, data=None, headers=None, auth=None,
                     verify=None):
-            assert method == 'OPTIONS'
             assert url == args['url']
             calls.append(None)
 
@@ -239,21 +199,35 @@ class TestCaldavStorage(DavStorageTests):
 
         monkeypatch.setattr('requests.sessions.Session.request', request)
 
-        with pytest.raises(vdirsyncer.exceptions.StorageError):
-            self.storage_class(**args)
+        with pytest.raises(ValueError):
+            s = self.storage_class(**args)
+            list(s.list())
         assert len(calls) == 1
 
-    def test_empty_get_multi_performance(self, monkeypatch):
-        s = self._get_storage()
-
-        def breakdown(*a, **kw):
-            raise AssertionError('Expected not to be called.')
-
-        monkeypatch.setattr('requests.sessions.Session.request', breakdown)
-
-        assert list(s.get_multi([])) == []
-
 
 class TestCarddavStorage(DavStorageTests):
     storage_class = CarddavStorage
     item_template = VCARD_TEMPLATE
+
+
+@pytest.mark.parametrize('base,path', [
+    ('http://example.com/', ''),
+    ('http://example.com/L%C3%98/', '/L%C3%98'),
+    ('http://example.com/LØ/', '/L%C3%98'),
+])
+def test_normalize_href(base, path):
+    assert _normalize_href(base, 'asdf') == path + '/asdf'
+
+    assert _normalize_href(base, 'hahah') == path + '/hahah'
+
+    assert _normalize_href(base, 'whoops@vdirsyncer.vcf') == \
+        path + '/whoops@vdirsyncer.vcf'
+
+    assert _normalize_href(base, 'whoops%40vdirsyncer.vcf') == \
+        path + '/whoops@vdirsyncer.vcf'
+
+    assert _normalize_href(base, 'wh%C3%98ops@vdirsyncer.vcf') == \
+        path + '/wh%C3%98ops@vdirsyncer.vcf'
+
+    assert _normalize_href(base, 'whØops@vdirsyncer.vcf') == \
+        path + '/wh%C3%98ops@vdirsyncer.vcf'
