@@ -12,9 +12,8 @@ import json
 import os
 import sys
 
-import click
-
 from . import __version__, log
+from .doubleclick import click
 from .storage import storage_names
 from .sync import StorageEmpty, SyncConflict, sync
 from .utils import expand_path, get_class_init_args, parse_options, split_dict
@@ -31,6 +30,9 @@ cli_logger = log.get(__name__)
 PROJECT_HOME = 'https://github.com/untitaker/vdirsyncer'
 DOCS_HOME = 'https://vdirsyncer.readthedocs.org/en/latest'
 
+GENERAL_ALL = set(['processes', 'status_path'])
+GENERAL_REQUIRED = set(['status_path'])
+
 
 class CliError(RuntimeError):
     pass
@@ -42,6 +44,28 @@ def get_status_name(pair, collection):
     return pair + '/' + collection
 
 
+def validate_general_section(general_config):
+    if general_config is None:
+        raise CliError(
+            'Unable to find general section. You should copy the example '
+            'config from the repository and edit it.\n{}'.format(PROJECT_HOME)
+        )
+
+    invalid = set(general_config) - GENERAL_ALL
+    missing = GENERAL_REQUIRED - set(general_config)
+
+    if invalid:
+        cli_logger.critical(u'general section doesn\'t take the parameters: {}'
+                            .format(u', '.join(invalid)))
+
+    if missing:
+        cli_logger.critical(u'general section is missing the parameters: {}'
+                            .format(u', '.join(missing)))
+
+    if invalid or missing:
+        raise CliError('Invalid general section.')
+
+
 def load_config(fname, pair_options=('collections', 'conflict_resolution')):
     c = RawConfigParser()
     with open(fname) as f:
@@ -80,12 +104,7 @@ def load_config(fname, pair_options=('collections', 'conflict_resolution')):
         else:
             handlers.get(section_type, bad_section)(name, get_options(section))
 
-    if general is None:
-        raise CliError(
-            'Unable to find general section. You should copy the example '
-            'config from the repository and edit it.\n{}'.format(PROJECT_HOME)
-        )
-
+    validate_general_section(general)
     return general, pairs, storages
 
 
@@ -120,11 +139,11 @@ def storage_class_from_config(config):
     storage_name = config.pop('type')
     cls = storage_names.get(storage_name, None)
     if cls is None:
-        raise KeyError('Unknown storage: {}'.format(storage_name))
+        raise CliError('Unknown storage type: {}'.format(storage_name))
     return cls, config
 
 
-def storage_instance_from_config(config, description=None):
+def storage_instance_from_config(config):
     '''
     :param config: A configuration dictionary to pass as kwargs to the class
         corresponding to config['type']
@@ -136,26 +155,33 @@ def storage_instance_from_config(config, description=None):
     try:
         return cls(**config)
     except Exception:
-        all, required = get_class_init_args(cls)
-        given = set(config)
-        missing = required - given
-        invalid = given - all
+        handle_storage_init_error(cls, config)
 
-        if not missing and not invalid:
-            cli_logger.exception('')
 
-        if missing:
-            cli_logger.critical(
-                u'error: {} storage requires the parameters: {}'
-                .format(cls.storage_name, u', '.join(missing)))
+def handle_storage_init_error(cls, config):
+    e = sys.exc_info()[1]
+    if isinstance(e, (click.Abort, CliError, KeyboardInterrupt)):
+        raise
 
-        if invalid:
-            cli_logger.critical(
-                u'error: {} storage doesn\'t take the parameters: {}'
-                .format(cls.storage_name, u', '.join(invalid)))
+    all, required = get_class_init_args(cls)
+    given = set(config)
+    missing = required - given
+    invalid = given - all
 
-        raise CliError('error: Failed to initialize {}'
-                       .format(description or cls.storage_name))
+    if missing:
+        cli_logger.critical(
+            u'{} storage requires the parameters: {}'
+            .format(cls.storage_name, u', '.join(missing)))
+
+    if invalid:
+        cli_logger.critical(
+            u'{} storage doesn\'t take the parameters: {}'
+            .format(cls.storage_name, u', '.join(invalid)))
+
+    if not missing and not invalid:
+        cli_logger.exception('')
+
+    raise CliError('Failed to initialize {}.'.format(config['instance_name']))
 
 
 def expand_collection(pair, collection, all_pairs, all_storages):
@@ -181,9 +207,13 @@ def expand_collection(pair, collection, all_pairs, all_storages):
         else:
             config.update(all_storages[b_name])
         cls, config = storage_class_from_config(config)
-        return (s.collection for s in cls.discover(**config))
+        try:
+            for s in cls.discover(**config):
+                yield s.collection
+        except Exception:
+            handle_storage_init_error(cls, config)
     else:
-        return [collection]
+        yield collection
 
 
 def parse_pairs_args(pairs_args, all_pairs):
@@ -262,7 +292,7 @@ def _create_app():
             try:
                 ctx.obj['config'] = load_config(fname)
             except Exception as e:
-                raise CliError('Error during reading config{}:\n{}'
+                raise CliError('Error during reading config{}: {}'
                                .format(fname, e))
 
     @app.command()
@@ -320,13 +350,14 @@ def _create_app():
         cli_logger.debug('Using {} processes.'.format(processes))
 
         if processes == 1:
-            cli_logger.debug('Not using multiprocessing.')
+            cli_logger.debug('Not using threads.')
             rv = (_sync_collection(x) for x in actions)
         else:
-            cli_logger.debug('Using multiprocessing.')
-            from multiprocessing import Pool
+            cli_logger.debug('Using threads.')
+            from multiprocessing.dummy import Pool
             p = Pool(processes=general.get('processes', 0) or len(actions))
-            rv = p.map_async(_sync_collection, actions).get(10**9)
+
+            rv = p.imap_unordered(_sync_collection, actions)
 
         if not all(rv):
             sys.exit(1)
@@ -347,13 +378,15 @@ def sync_collection(config_a, config_b, pair_name, collection, pair_options,
     collection_description = pair_name if collection is None \
         else '{} from {}'.format(collection, pair_name)
 
-    a = storage_instance_from_config(config_a, collection_description)
-    b = storage_instance_from_config(config_b, collection_description)
-
-    cli_logger.info('Syncing {}'.format(collection_description))
-    status = load_status(general['status_path'], status_name)
     rv = True
     try:
+        cli_logger.info('Syncing {}'.format(collection_description))
+
+        a = storage_instance_from_config(config_a)
+        b = storage_instance_from_config(config_b)
+
+        status = load_status(general['status_path'], status_name)
+        cli_logger.debug('Loaded status for {}'.format(collection_description))
         sync(
             a, b, status,
             conflict_resolution=pair_options.get('conflict_resolution', None),
@@ -384,10 +417,13 @@ def sync_collection(config_a, config_b, pair_name, collection, pair_options,
             'Item href on side B: {e.href_b}\n'
             .format(collection=collection_description, e=e, docs=DOCS_HOME)
         )
-    except Exception:
+    except (click.Abort, KeyboardInterrupt):
+        rv = False
+    except Exception as e:
         rv = False
         cli_logger.exception('Unhandled exception occured while syncing {}.'
                              .format(collection_description))
 
-    save_status(general['status_path'], status_name, status)
+    if rv:
+        save_status(general['status_path'], status_name, status)
     return rv
