@@ -36,7 +36,14 @@ class Storage(object):
     '''
 
     fileext = '.txt'
-    storage_name = None  # The name used in the config file.
+
+    # The string used in the config to denote the type of storage. Should be
+    # overridden by subclasses.
+    storage_name = None
+
+    # The string used in the config to denote a particular instance. Should be
+    # overridden during instantiation.
+    instance_name = None
 
     # A value of True means the storage does not support write-methods such as
     # upload, update and delete.  A value of False means the storage does
@@ -46,12 +53,13 @@ class Storage(object):
     # The attribute values to show in the representation of the storage.
     _repr_attributes = ()
 
-    def __init__(self, read_only=None):
+    def __init__(self, instance_name=None, read_only=None):
         if read_only is None:
             read_only = self.read_only
         if self.read_only and not read_only:
             raise ValueError('This storage is read-only.')
         self.read_only = bool(read_only)
+        self.instance_name = instance_name
 
     @classmethod
     def discover(cls, **kwargs):
@@ -66,7 +74,7 @@ class Storage(object):
         raise NotImplementedError()
 
     def __repr__(self):
-        return '<{}(**{})>'.format(
+        return self.instance_name or '<{}(**{})>'.format(
             self.__class__.__name__,
             dict((x, getattr(self, x)) for x in self._repr_attributes)
         )
