@@ -25,24 +25,33 @@ sync_logger = log.get(__name__)
 class SyncError(exceptions.Error):
     '''Errors related to synchronization.'''
 
-    def __init__(self, *args, **kwargs):
-        self.__dict__.update(kwargs)
-        super(SyncError, self).__init__(*args)
-
 
 class SyncConflict(SyncError):
     '''
     Two items changed since the last sync, they now have different contents and
     no conflict resolution method was given.
+
+    :param ident: The ident of the item.
+    :param href_a: The item's href on side A.
+    :param href_b: The item's href on side B.
     '''
 
+    ident = None
+    href_a = None
+    href_b = None
+
 
 class StorageEmpty(SyncError):
     '''
     One storage unexpectedly got completely empty between two synchronizations.
     The first argument is the empty storage.
+
+    :param empty_storage: The empty
+        :py:class:`vdirsyncer.storage.base.Storage`.
     '''
 
+    empty_storage = None
+
 
 class BothReadOnly(SyncError):
     '''
@@ -64,13 +73,22 @@ def prepare_list(storage, href_to_status):
         else:
             download.append(href)
 
-    if download:
-        for href, item, etag in storage.get_multi(download):
-            props = rv[href]
-            props['item'] = item
-            props['ident'] = item.ident
-            if props['etag'] != etag:
-                raise SyncError('Etag changed during sync.')
+    prefetch(storage, rv, download)
+    return rv
+
+
+def prefetch(storage, rv, hrefs):
+    if rv is None:
+        rv = {}
+    if not hrefs:
+        return rv
+
+    for href, item, etag in storage.get_multi(hrefs):
+        props = rv.setdefault(href, {'etag': etag})
+        props['item'] = item
+        props['ident'] = item.ident
+        if props['etag'] != etag:
+            raise SyncError('Etag changed during sync.')
 
     return rv
 
@@ -95,8 +113,9 @@ def sync(storage_a, storage_b, status, conflict_resolution=None,
         safety. Setting this parameter to ``True`` disables this safety
         measure.
     '''
-    if False not in (storage_a.read_only, storage_b.read_only):
+    if storage_a.read_only and storage_b.read_only:
         raise BothReadOnly()
+
     a_href_to_status = dict(
         (href_a, (ident, etag_a))
         for ident, (href_a, etag_a, href_b, etag_b) in iteritems(status)
@@ -127,7 +146,9 @@ def sync(storage_a, storage_b, status, conflict_resolution=None,
         action(storages, status, conflict_resolution)
 
 
-def action_upload(ident, source, dest):
+def action_upload(ident, dest):
+    source = 'a' if dest == 'b' else 'b'
+
     def inner(storages, status, conflict_resolution):
         source_storage, source_list, source_ident_to_href = storages[source]
         dest_storage, dest_list, dest_ident_to_href = storages[dest]
@@ -154,7 +175,9 @@ def action_upload(ident, source, dest):
     return inner
 
 
-def action_update(ident, source, dest):
+def action_update(ident, dest):
+    source = 'a' if dest == 'b' else 'b'
+
     def inner(storages, status, conflict_resolution):
         source_storage, source_list, source_ident_to_href = storages[source]
         dest_storage, dest_list, dest_ident_to_href = storages[dest]
@@ -224,12 +247,10 @@ def action_conflict_resolve(ident):
             raise SyncConflict(ident=ident, href_a=href_a, href_b=href_b)
         elif conflict_resolution == 'a wins':
             sync_logger.info('...{} wins.'.format(a_storage))
-            action_update(ident, 'a', 'b')(storages, status,
-                                           conflict_resolution)
+            action_update(ident, 'b')(storages, status, conflict_resolution)
         elif conflict_resolution == 'b wins':
             sync_logger.info('...{} wins.'.format(b_storage))
-            action_update(ident, 'b', 'a')(storages, status,
-                                           conflict_resolution)
+            action_update(ident, 'a')(storages, status, conflict_resolution)
         else:
             raise ValueError('Invalid conflict resolution mode: {}'
                              .format(conflict_resolution))
@@ -255,18 +276,18 @@ def get_actions(storages, status):
             if a and b:  # missing status
                 yield action_conflict_resolve(ident)
             elif a and not b:  # new item was created in a
-                yield action_upload(ident, 'a', 'b')
+                yield action_upload(ident, 'b')
             elif not a and b:  # new item was created in b
-                yield action_upload(ident, 'b', 'a')
+                yield action_upload(ident, 'a')
         else:
             _, status_etag_a, _, status_etag_b = status[ident]
             if a and b:
                 if a['etag'] != status_etag_a and b['etag'] != status_etag_b:
                     yield action_conflict_resolve(ident)
                 elif a['etag'] != status_etag_a:  # item was updated in a
-                    yield action_update(ident, 'a', 'b')
+                    yield action_update(ident, 'b')
                 elif b['etag'] != status_etag_b:  # item was updated in b
-                    yield action_update(ident, 'b', 'a')
+                    yield action_update(ident, 'a')
             elif a and not b:  # was deleted from b
                 yield action_delete(ident, 'a')
             elif not a and b:  # was deleted from a
